/*
   Copyright (C) 2006-2009
   by Marco Gulino <marco.gulino@gmail.com>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License version 2.1 as published by the Free Software Foundation.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
*/
#include "togglepanel.h"
#include <QPainter>
#include <QFontMetrics>
#include <QSizeF>
#include <KLocale>

#include <plasma/svg.h>
#include <plasma/theme.h>
#include <plasma/containment.h>
#include <plasma/corona.h>
#include <kdebug.h>
#include <kconfigdialog.h>
#include <QHBoxLayout>
#include <QLabel>
#include <kcombobox.h>
#include <Plasma/IconWidget>
#include <QGraphicsLinearLayout>
#include <QGraphicsView>
#include <kwindowsystem.h>
#include <kwindowinfo.h>

#define HIDE_OWN_PANEL true

TogglePanel::TogglePanel(QObject *parent, const QVariantList &args)
    : Plasma::Applet(parent, args), kcfg_combo_panel(NULL), currentPanel(0)
{
    // this will get us the standard applet background, for free!
    setBackgroundHints(DefaultBackground);
//     m_svg.setImagePath("widgets/background");
    setHasConfigurationInterface(true);  

    resize(200, 200);
}


TogglePanel::~TogglePanel()
{
    if (hasFailedToLaunch()) {
        // Do some cleanup here
    } else {
        // Save settings
    }
}

void TogglePanel::init()
{
  currentPanel=config().readEntry<uint>("currentPanel", 0);
  setConfigurationRequired(currentPanel==0);
  QGraphicsLinearLayout *layout=new QGraphicsLinearLayout(Qt::Horizontal, this);
  Plasma::IconWidget *iconWidget = new Plasma::IconWidget(KIcon("plasma"),"", this) ;
  layout->addItem(iconWidget);
  iconWidget->setVisible(true);
  connect(iconWidget, SIGNAL(clicked()), this, SLOT(togglePanel()));
  connect(this, SIGNAL(activate()), this, SLOT(togglePanel()));
}


void TogglePanel::createConfigurationInterface(KConfigDialog* parent)
{
//     delete kcfg_combo_panel;
    QWidget *widget=new QWidget(parent);
    QHBoxLayout *layout=new QHBoxLayout(widget);
    layout->addWidget(new QLabel(i18n("Panel to Show/Hide")));
    kcfg_combo_panel = new KComboBox();
    layout->addWidget(kcfg_combo_panel);
    QHash<uint, Plasma::Containment*> panels = getPanels(HIDE_OWN_PANEL);
    foreach(uint key, panels.keys() ) {
      kcfg_combo_panel->addItem(locationToString( panels[key]->location() ),key );
    }
    parent->addPage(widget, i18n("Panel Configuration"));
    connect(parent, SIGNAL(applyClicked()), this, SLOT(configured()));
    connect(parent, SIGNAL(okClicked()), this, SLOT(configured()));

}



void TogglePanel::togglePanel()
{
  kDebug() << "Toggle Panel: " << currentPanel << endl;
  Plasma::Containment *panel=getPanels(HIDE_OWN_PANEL)[currentPanel];
  uint winId=panel->view()->winId();
  if(!panel || configurationRequired() ) {
    kDebug() << "Panel not found, aborting" << endl;
    return;
  }
  bool wasVisible=panel->view()->isVisible();
  if(wasVisible) {
    KWindowInfo oldInfo=KWindowSystem::windowInfo(winId, NET::WMState );
    oldState=oldInfo.state();
    kDebug() << "got old state: " << oldState << endl;
    panel->view()->setVisible(false);
  }
  else {
    panel->view()->setVisible(true);
    KWindowSystem::clearState(winId, NET::KeepAbove);
    KWindowSystem::setState(winId, oldState | NET::StaysOnTop);
    KWindowSystem::setOnAllDesktops(winId, true);
  }
}


void TogglePanel::configured()
{
    bool changed = false;
    KConfigGroup cg = config();

    if (currentPanel != kcfg_combo_panel->itemData(kcfg_combo_panel->currentIndex() )) {
        currentPanel = kcfg_combo_panel->itemData(kcfg_combo_panel->currentIndex() ).toUInt();
        cg.writeEntry("currentPanel", currentPanel);
        changed = true;
    }

    if (changed) {
        emit configNeedsSaving();
    }
    setConfigurationRequired(currentPanel==0);
}



QHash<uint, Plasma::Containment*> TogglePanel::getPanels(bool excludeMine)
{
    QHash<uint, Plasma::Containment*> panels;
    foreach(Plasma::Containment* panel, containment()->corona()->containments()) {
      if(panel->containmentType() == Plasma::Containment::PanelContainment
	&& ( (!excludeMine) || panel->id() != containment()->id() )
	)
	panels.insert(panel->id(), panel);
    }
    setConfigurationRequired(! panels.keys().contains(currentPanel));
    return panels;
}

QString TogglePanel::locationToString(Plasma::Location location) const
{
  switch(location) {
    case Plasma::TopEdge:
       return i18n("Top Edge");
    case Plasma::BottomEdge:
      return i18n("Bottom Edge");
    case Plasma::LeftEdge:
      return i18n("Left Edge");
    case Plasma::RightEdge:
      return i18n("Right Edge");
    default:
      return i18n("Unknown Position (%1)",location);
  }
}

#include "togglepanel.moc"
